from textual.app import App, ComposeResult
from textual.containers import ScrollableContainer, Horizontal
from textual.widgets import DataTable, Static
from textual.binding import Binding
from datetime import datetime

from ..controller.summaries import (
    get_category_summary,
    get_recently_active,
    get_due_soon,
)


class CategoryTable(DataTable):
    """Table showing category summaries"""

    def on_mount(self):
        self.add_columns(
            "Category", "Zero", "WIP", "Blocked", "Done", "Overdue", "Due Soon"
        )
        self.refresh_data()

    def refresh_data(self):
        self.clear()
        summaries = get_category_summary(10)  # Show top 10 categories
        for summary in summaries:
            self.add_row(
                summary["category"],
                str(summary["tasks"]["zero"]),
                str(summary["tasks"]["wip"]),
                str(summary["tasks"]["blocked"]),
                str(summary["tasks"]["done"]),
                str(summary["tasks"]["overdue"]),
                str(summary["tasks"]["due_soon"]),
            )


class TaskList(DataTable):
    """Base class for task list tables"""

    def on_mount(self):
        self.add_columns("Task", "Status", "Category", "Due Date")

    def format_due_date(self, due_date: datetime | None) -> str:
        if not due_date:
            return "No due date"
        return due_date.strftime("%Y-%m-%d")


class DueTaskList(TaskList):
    """Table showing upcoming and overdue tasks"""

    def on_mount(self):
        super().on_mount()
        self.refresh_data()

    def refresh_data(self):
        self.clear()
        tasks = get_due_soon(10, all_overdue=True)  # Show all overdue + next 10
        for task in tasks:
            self.add_row(
                task["text"],
                task["status"],
                task["category"] or "No category",
                self.format_due_date(task["due"]),
                key=str(task["id"]),
            )


class RecentTaskList(TaskList):
    """Table showing recently active tasks"""

    def on_mount(self):
        super().on_mount()
        self.refresh_data()

    def refresh_data(self):
        self.clear()
        tasks = get_recently_active(10)  # Show 10 most recent
        for task in tasks:
            self.add_row(
                task["text"],
                task["status"],
                task["category"] or "No category",
                self.format_due_date(task["due"]),
                key=str(task["id"]),
            )


class Overview(App):
    """Task overview application"""

    TITLE = "Task Overview"
    CSS = """
    CategoryTable {
        height: 40%;
        margin: 1 1;
    }
    
    #lists {
        height: 60%;
    }
    
    TaskList {
        width: 50%;
        margin: 1 1;
    }
    
    Static {
        content-align: center middle;
        background: $panel;
        padding: 1;
    }
    """

    BINDINGS = [
        Binding("q", "quit", "Quit"),
        Binding("r", "refresh", "Refresh Data"),
    ]

    def compose(self) -> ComposeResult:
        yield CategoryTable()
        yield Static("Upcoming and Recent Tasks")
        with Horizontal(id="lists"):
            with ScrollableContainer():
                yield DueTaskList()
            with ScrollableContainer():
                yield RecentTaskList()

    def action_refresh(self):
        """Refresh all data in the tables"""
        self.query_one(CategoryTable).refresh_data()
        self.query_one(DueTaskList).refresh_data()
        self.query_one(RecentTaskList).refresh_data()


if __name__ == "__main__":
    app = Overview()
    app.run()
